/*
 *	eXtended-Library
 *	released for GNU-C
 *	by Volker Hemsen, 03/96, 04/96, 05/96, 06/96, 07/96, 01/97
 *	11/97, 11/99
 *	Public Domain / Freeware
 */

#ifndef _XLB_H
#define _XLB_H

#ifdef __cplusplus
extern "C" {
#endif

#include <macros.h>
#include <string.h>
#include <stdio.h>

#ifdef LATTICE
#include <aes.h>
#elif defined(__GNUC__)
/* verwende eine neue GEM-Lib */
#define _XLB_GEMLIB37
#ifdef _XLB_GEMLIB37
#include <gem.h>
#else
#include <gemfast.h>
#include <aesbind.h>
#include <vdibind.h>
#endif
#endif

#if (!defined(size_t) && !defined(_SIZE_T))
#define size_t long
#endif

#ifndef NULL
#define NULL ((void*)0)
#endif

typedef enum { FALSE, TRUE } boolean;

/***************************************************
 **   Lattice C 5.5 - compatible STRING-Library   **
 ***************************************************/

#if !defined(LATTICE) && !defined(__STRICT_ANSI__)
#define stccpy strcpy
#define stclen strlen
extern void strins(char* to, const char* from);
extern int strmid(char* src, char* dst, size_t pos, size_t len);
extern void strdel(char* s, int pos, int len);
extern char *strlastp(char *s);
extern char strlast(char* s);
extern void strcatc(char* dst, char c);
extern void strcatcs(char* dst, char c, char* src);
extern char *strtrim(char *s);
extern char *strupper(char *s);
extern char *strlower(char *s);
extern char *strto7bit(char *s);
extern int stcgfe(char* ext, char* name);
extern int stcgfn(char* node, char* name);
extern int stcgfp(char* path, char* name);
extern int stcwoe(char* namewoe, char* name);
extern void strmfe(char* newname, char* oldname, char* ext);
extern void strmfp(char* name, char* path, char* node);
extern void strmfn(char* file,char* drive,char* path,char* node,char* ext);
extern void strsfn(char* file, char* drive, char* path, char* node, char* ext);
extern size_t strtst(char *s);
extern long strpladd(char *s[], size_t nmax, char *t);
extern long strbpl(char *s[], size_t nmax, char *t);
extern void strsrt(char *s[], size_t n);
extern char *stpblk(char *p);
extern char *stpsym(char *s, char *sym, size_t symlen);
extern int stcd_i(char *in, int *ivalue);
extern int stch_i(char *in, int *ivalue);
extern int stcd_l(char *in, long *lvalue);
extern int stch_l(char *in, long *lvalue);
extern int stci_d(char *out, int ivalue);
extern int stcu_d(char *out, unsigned int ivalue);
extern int stci_h(char *out, int ivalue);
extern int stcl_d(char *out, long lvalue);
extern int stcul_d(char *out, unsigned long lvalue);
extern int stcl_h(char *out, long lvalue);
#endif /* !defined(LATTICE) && !defined(__STRICT_ANSI__) */
#define strclr(s) *s='\0'
extern int stci_r(char *out, int ivalue);
extern int stcd_de(char *in, double *dvalue);
extern int stcde_d(char *out, double dvalue);
extern boolean _STCXD_NORM;
extern int stcxd_d(char *in, double *dvalue);
extern int stcd_xd(char *out, double dvalue);
extern int stcl_a(long val,char *s);
extern int stcll_a(long long val,char *s);
extern double parser(char *);
extern char *xgetenv(const char *val);
#define getenv(a) xgetenv(a)


/******************************
 **   Memory-Field_Library   **
 ******************************/

#ifndef LATTICE
extern void memswp(void *a, void *b, long len);
#define swmem memswp
#define setmem memset
#endif
extern void* malloc_global(long len);
#define memclr(a,l) memset((a),0,(l))

typedef struct field_structure {
	void*	address;	/* field address */
	long	size;		/* size of one element */
	long	usedmax;	/* last used element */
	long	memmax;		/* numbers of allocated */
	long	addmem;		/* add element at allocation */
} FIELD;

extern FIELD *FIELD_init(long size, long addmem);
extern void FIELD_end(FIELD **f);
extern char *FIELD_address(FIELD *f, long pos);
extern void FIELD_free(FIELD *f);
extern boolean FIELD_allocate(FIELD *f, long newmax);
extern boolean FIELD_delete(FIELD *f, long pos);
extern boolean FIELD_insert(FIELD *f, long pos, void *insert);
extern boolean FIELD_add(FIELD *f, void *insert);
extern boolean FIELD_sort(FIELD *f, long *curs, boolean (*routine )(FIELD *f,long i1, long i2 ));


/***************************
 **   Cookie-Behandlung   **
 ***************************/
#define _AKP	0x5f414b50L		/* '_AKP' */
#define _CPU	0x5f435055L		/* '_CPU' */
#define _CPUC	0x43505543L		/* 'CPUC' */
#define _FDC	0x5f464443L		/* '_FDC' */
#define _FLK	0x5f464c4bL		/* '_FLK' */
#define _FPU	0x5f465055L		/* '_FPU' */
#define _FPUC	0x46505543L		/* 'CPUC' */
#define _FRB	0x5f465242L		/* '_FRB' */
#define _Gnva	0x476e7661L		/* 'Gnva' */
#define _HDJR	0x48444a52L		/* 'HDJR' */
#define _ICFS	0x49434653L		/* 'ICFS' */
#define _IMNE	0x494d4e45L
#define _MAGX	0x4D616758L		/* `MagX' */
#define _MCH	0x5f4d4348L
#define _MINT	0x4d694e54L		/* 'MiNT' */
#define _MiNT	_MINT
#define _MNAM	0x4d4e414dL
#define _NVDI	0x4e564449L
#define _NET	0x5f4e4554L
#define _SND	0x5f534e44L
#define _SWI	0x5f535749L
#define _VDO	0x5f56444fL
#define _VSCR	0x56534352L		/* `VSCR' */
#define _XBRA	0x58425241L		/* `XBRA' */
#define _XHDI	0x58484449L
#define _T2GM	0x5432474DL		/* 'T2GM' */
extern boolean getcookie(long,long*);
#define get_cookie(a,b) getcookie(a,b)
extern boolean putcookie(long cookie,long value);
#define put_cookie(a,b) putcookie(a,b)
extern boolean rmvcookie(long cookie);


/******************************
 **   extended DOS-Library   **
 ******************************/
#ifndef _STDIO_H
# define FILE void
#endif
#ifndef PATH_MAX
# define PATH_MAX 256
#endif
#ifndef FILENAME_MAX
# define FILENAME_MAX 128
#endif
#ifndef NAME_MAX
# define NAME_MAX 32
#endif
extern int mkdir(char *path);
extern int rmdir(char *path);
extern short getdrive(void);
extern void setdrive(short drive);
extern void xgetpath(short drive,char* path);
#define getpath(a)	xgetpath(-1,a)
extern char *getcwd(char *b,size_t size);
extern void setpath(char* path);
#define putcwd(a) setpath(a)
extern long drives(void);
extern boolean tstdrive(short dr);
extern boolean tstdrivec(char c);
/*extern short fgetw(FILE *fp);
extern long fgetl(FILE *fp);
extern short fputw(short val, FILE *fp);
extern long fputl(long val, FILE *fp);*/
#if !defined(__STRICT_ANSI__) && !defined(_POSIX_SOURCE)
#define fgetw(a)	getw(a)
#define	fgetl(a)	getl(a)
#define	fputw(b,a)	putw(b,a)
#define	fputl(b,a)	putl(b,a)
#endif
extern char *syserr(long errnr);
extern short syslan(void);
extern void Coutchar(char c);
extern void Coutbel(void);
extern void Coutesc(char c);
extern void Coutstr(char *s);
extern boolean filecopy(char *fromfile, char *tofile);
extern boolean filemove(char *fromfile, char *tofile);
extern boolean fileexist(char *name);
extern FILE *fopene(char *name, char *mode, char *path);
extern int flushall(void);
extern int fcloseall(void);
extern short rands(void);
extern long rand24(void);
extern long randl(void);
extern boolean randb(void);
extern long Fputs(char *s, int handle);
extern long Fputc(char c, int handle);
extern long Fputw(short w, int handle);
extern long Fputl(long l, int handle);
extern long Fgetc(int handle);
extern char *Fgets(char *buffer, int length, int handle);
extern short Fgetw(int handle);
extern long Fgetl(int handle);
extern int Fgetpos(int handle, long *pos);
extern int Fsetpos(int handle, long *pos);
extern boolean isatty(int handle);
extern void getargvopt(int argc,char *argv[],boolean routine(char *c));
extern boolean initterm(void);
extern char *tgetstrx(char *id, char *area);
#define Pabsnice(pri)	{ register int p=(Pnice(0)-pri); Pnice(p); }

/******************************
 **   extended AES-Library   **
 ******************************/

/* messages */
#define WM_TOP WM_TOPPED
#define WM_NEWTOP		29	/* since GEM\2 */
#define WM_UNTOPPED		30	/* since GEM\2, AES 4.0 */
#define WM_UNTOP WM_UNTOPPED
#define WM_ONTOP		31	/* since AES 4.0 */
#define WM_BOTTOMED		33	/* since AES 4.0 */
#define WM_BOTTOM WM_BOTTOMED
#define WM_ICONIFY		34	/* since AES 4.1 */
#define WM_UNICONIFY	35	/* since AES 4.1 */
#define WM_ALLICONIFY	36	/* since AES 4.1 */
#define WM_TOOLBAR		37	/* since AES 4.1 */
#define AP_TERM			50	/* since AES 4.1 */
#define AP_TFAIL		51	/* since AES 4.1 */
#define CT_KEY			53	/* XControl */
#define AP_RESCHG		57
#define SHUT_COMPLETED	60
#define RESCH_COMPLETED	61
#define	AP_DRAGDROP		63	/* since AES 4.0 */
#define SH_WDRAW		72
#define SC_CHANGED		80	/* Clipboard */
#define CH_EXIT			90	/* Child terminated */
#define WM_M_BDROPPED	100	/* since ??? */
#define SM_M_SPECIAL	101	/* MagiC */
#define WM_SHADED	22360	/* ?? */
#define WM_UNSHADED	22361	/* ?? */

#ifndef __vaproto__
#define ACC_ID			0x400
#define ACC_OPEN		0x401
#define ACC_CLOSE		0x402
#define ACC_ACC			0x403
#define AV_PROTOKOLL		0x4700
#define VA_PROTOSTATUS		0x4701
#define AV_GETSTATUS		0x4703
#define AV_STATUS			0x4704
#define VA_SETSTATUS		0x4705
#define	AV_SENDKEY			0x4710
#define VA_START			0x4711
#define AV_ASKFILEFONT		0x4712
#define VA_FILEFONT			0x4713
#define AV_ASKCONFONT		0x4714
#define VA_CONFONT			0x4715
#define AV_ASKOBJECT		0x4716
#define VA_OBJECT			0x4717
#define AV_OPENCONSOLE		0x4718
#define VA_CONSOLEOPEN		0x4719
#define AV_OPENWIND			0x4720
#define VA_WINDOPEN			0x4721
#define AV_STARTPROG		0x4722
#define VA_PROGSTART		0x4723
#define AV_ACCWINDOPEN		0x4724
#define VA_DRAGACCWIND		0x4725
#define AV_ACCWINDCLOSED	0x4726
#define AV_COPY_DRAGGED		0x4728
#define VA_COPY_COMPLETE	0x4729
#define AV_PATH_UPDATE		0x4730
#define AV_WHAT_IZIT		0x4732
#define	VA_OB_UNKNOWN	0		
#define VA_OB_TRASHCAN  1
#define VA_OB_SHREDDER  2
#define VA_OB_CLIPBOARD 3
#define VA_OB_FILE      4
#define VA_OB_FOLDER	5
#define VA_OB_DRIVE		6
#define VA_OB_WINDOW    7
#define VA_THAT_IZIT		0x4733
#define AV_DRAG_ON_WINDOW	0x4734
#define VA_DRAG_COMPLETE	0x4735
#define AV_EXIT				0x4736
#define	AV_STARTED			0x4738
#define	VA_FONTCHANGED		0x4739
#define	AV_XWIND			0x4740
#define	VA_XOPEN			0x4741
#endif

/* wind_set() / wind_get()	*/
#define WF_OWNER		20
#define WF_BEVENT		24	/* since AES 4.0 */
#define	WF_BOTTOM		25	/* since AES 4.0 */
#define WF_ICONIFY		26	/* since AES 4.1 */
#define	WF_UNICONIFY	27	/* since AES 4.1 */
#define WF_UNICONIFYXYWH	28	/* since AES 4.1 */

/* window attributs */
#ifdef __GNUC__
# define MOVE	MOVER
# define CLOSE	CLOSER
# define FULL	FULLER
#endif
#define ALLHEAD (NAME|FULL|INFO|CLOSE|FULL|MOVE)
#define VSLIDER	(UPARROW|DNARROW|VSLIDE)
#define HSLIDER	(LFARROW|RTARROW|HSLIDE)
#define ALLKIND	(ALLHEAD|VSLIDER|HSLIDER|SIZER)
#ifndef _XLB_GEMLIB37
#define BACKDROP	0x2000;	/* since ??? */
#define SMALLER		0x4000	/* since AES 4.1 */
#endif

/* D&D-messages */
#define DD_OK	0
#define DD_NAK	1
#define	DD_EXT	2
#define	DD_LEN	3
#define DD_TRASH	4
#define DD_PRINTER	5
#define DD_CLIPBOARD	6
#define NO_DD		-100
#define NO_RECEIVER	-101

/* which from objc_sysvar() */
#define LK3DIND		1
#define LK3DACT		2
#define INDBUTCOL	3
#define ACTBUTCOL	4
#define BACKGRCOL	5
#define AD3DVALUE	6

/* mouse */
#define BUSYBEE BEE

typedef struct
{
	short flags;
	short bclk,bmsk,bst;
	short m1flags,m1x,m1y,m1w,m1h;
	short m2flags,m2x,m2y,m2w,m2h;
	short msg[8];
	unsigned long time;
	short ev;
	short mx,my,mb;
	short ks,kr,br;
	unsigned short key;
}   MEVENT;

/* GRECT */
#ifndef _XLB_GEMLIB37
typedef struct graphic_rectangle
	{int g_x;int g_y;int g_w;int g_h;} GRECT;
#endif
extern void clr_grect(GRECT*);
extern boolean tst_grect(GRECT*);
extern GRECT* xywh_to_grect(int,int,int,int,GRECT*);
extern GRECT* pshort_to_grect(short *,GRECT *);
extern short* grect_to_pshort(GRECT*,short *);
extern GRECT* grect_add(GRECT*,int);
extern short* grect_to_sarray(GRECT*,short*);

/* AES-Calls */
#ifndef _XLB_GEMLIB37
extern int appl_getinfo(int typ,int *out1,int *out2,int *out3,int *out4);
extern int appl_xgetinfo(int typ,int *out1,int *out2,int *out3,int *out4);
extern int appl_search(int ap_smode,char *ap_sname,int *ap_stype,int *ap_sid);
extern int objc_sysvar(int mod,int which,int ival1,int ival2,
	int *oval1,int *oval2);
extern int rsrc_rcfix(void *rc_header);
#endif
extern boolean MultiTOS(void);
extern boolean has_iconify(void);
extern void tell_shutdown(void);
extern boolean has_bevent(void);
extern void wset_bevent(int handle,boolean mode);
extern boolean wget_bevent(int handle);
extern boolean has_xget9(short nr);
#define has_submenu		has_xget9(0)
#define has_popup		has_xget9(1)
#define has_scrollmenu	has_xget9(2)
#define has_extmnselected	has_xget9(3)
#define graf_rubbox		graf_rubberbox
#define wind_name		wind_title
extern int evnt_event(MEVENT *mevent);
extern boolean fileselector(char *ext, char *title, char *urpath);
#define fselbox fileselector
extern void begin_update(void);
extern void end_update(void);
extern void begin_mctrl(void);
#define begin_mouse begin_mctrl
extern void end_mctrl(void);
#define end_mouse end_mctrl
extern void clear_update(void);
extern void hide_mouse(void);
extern boolean hide_mouse_inside(GRECT *rect);
extern void show_mouse(void);
extern void arrow_mouse(void);
extern void busybee_mouse(void);
extern void hourglas_mouse(void);
extern short wind_top(void);
extern int aes_vhandle(void);
extern int aes_version(void);
extern void evnt_debutton(short mb);
extern void aeserror(void);
extern void wind_nodesk(void);
extern void getQSB(void** where,long* length);
extern void wind_gcalc(int wctype, int kind, const GRECT *rect1, GRECT *rect2);
extern boolean has_morewindows(void);
extern int wind_countrects(int handle);
extern boolean wind_seeall(int handle);
extern int wind_gcreate(int kind, GRECT *rect);
extern int wind_gopen(int handle, GRECT *rect);
extern void wind_gget(int handle, int field, GRECT *rect);
extern void wind_gset(int handle, int field, const GRECT *rect);
extern void wind_gcalc(int wctype, int kind, const GRECT *rect1, GRECT *rect2);
extern void objc_gdraw(OBJECT*,int,int,GRECT*);
extern void wind_xredraw(int handle,int vhandle,long par,
	GRECT *rect,GRECT* rectwork,void (*routine)(int handle,long par,GRECT *p));
extern void form_wredraw(int handle,GRECT *rect);
extern void wind_objcredraw(int handle,int vhandle,OBJECT *tree,int ob);
extern void wind_objcchangeredraw(int,int,OBJECT*,int,boolean);
extern void form_gdial(int flag,GRECT *rect1,GRECT *rect2);
extern void form_gcenter(OBJECT*,GRECT*);
extern void objc_gchange(OBJECT*,short,GRECT*,unsigned short);
#ifndef LATTICE
	extern void wind_title(int handle, char *title);
	extern void wind_info(int handle, char *info);
	extern void wind_newdesk(OBJECT *tree, int objc);
	extern void objc_xywh(OBJECT* tree,short objc,GRECT *rec);
	extern void wind_redraw(int handle,GRECT *rect,void (*routine)(int handle,GRECT *p));
	extern boolean rc_inside(int x,int y,GRECT *rect);
#endif
extern void drawobjc(OBJECT *tree, short objc);
extern void drawtree(OBJECT *tree, short objc);
extern void setobstr(OBJECT *tree, short objc, char *ob_str);
extern void setobint(OBJECT *tree, short objc, int val);
extern void getobstr(OBJECT *tree, short objc, char *ob_str);
extern int getobint(OBJECT *tree, short objc);
extern void setobselect(OBJECT *tree, short objc, boolean state);
extern void setobdefault(OBJECT *tree, short objc, boolean state);
extern boolean getobselect(OBJECT *tree, short objc);
extern void setobdisable(OBJECT *tree, short objc, boolean mode);
extern void gfbk_rubbox(short xor,short yor,short wmin,short hmin,short *wend,short *hend);
extern void gfbk_dragbox(short w,short h,short sx,short sy,
		short xc,short yc,short wc,short hc,short *dx,short *dy);
extern void gfbk_alldragbox(short w,short h,short sx,short sy,short *dx,short *dy);
extern boolean scrp_find(char *path);
#ifndef _XLB_GEMLIB37
extern boolean scrp_clear(void);
#endif
extern boolean scrp_init(char *ext,char *path);


/******************************
 **   extended VDI-Library   **
 ******************************/

#ifdef __GNUC__
#define FIS_HOLLOW	0
#define HOLLOW	FIS_HOLLOW
#define FIS_SOLID	1
#ifndef _XLB_GEMLIB37
#define SOLID	FIS_SOLID
#endif
#define FIS_PATTERN	2
#define PATTERN	FIS_PATTERN
#define FIS_HATCH	3
#define HATCH	FIS_HATCH
#define FIS_USER	4
#define UDFILLSTYLE	FIS_USER

#define ALL_WHITE	0	/* bit blt rules */
#define S_AND_D	1
#define S_AND_NOTD	2
#define S_ONLY	3
#define NOTS_AND_D	4
#define D_ONLY	5
#define S_XOR_D	6
#define S_OR_D	7
#define NOT_SORD	8
#define NOT_SXORD	9
#define NOT_D	10
#define S_OR_NOTD	11
#define NOT_S	12
#define NOTS_OR_D	13
#define NOT_SANDD	14
#define ALL_BLACK	15

/*#define SOLID	1*/	/* polyline line styles */
#define LDASHED	2
#define LONGDASH	LDASHED
#define DOTTED	3
#define DOT	DOTTED
#define DASHDOT	4
#define DASHED	5
#ifndef _XLB_GEMLIB37
#define DASH	DASHED
#endif
#define DASHDOTDOT	6
#define DASH2DOT	DASHDOTDOT
#define USERLINE	7

#define SQUARE	0
#define SQUARED	SQUARE
#define ARROWED	1
#define ROUND	2
#define ROUNDED	ROUND

#define DOTS	3	/* fill patterns */
#define GRID	6
#define BRICKS	9
#define WEAVE	16

#define THICKENED  0x0001	/* text effects */
#define SHADED     0x0002
#define SKEWED     0x0004
#define UNDERLINED 0x0008
#define OUTLINE    0x0010
#define SHADOW     0x0020
#define TF_NORMAL     0x00
#define TF_THICKENED  0x01
#define TF_LIGHTENED  0x02
#define TF_SLANTED    0x04
#define TF_UNDERLINED 0x08
#define TF_OUTLINED   0x10
#define TF_SHADOWED   0x20

#define MT_DOT		1		/* marker typs */
#define MT_PLUS		2
#define MT_ASTERISK	3
#define MT_SQUARE	4
#define MT_DCROSS	5
#define MT_DIAMOND	6
#define MT_DEVICE	7

#define V_WHITE		0		/* 16 colors */
#define V_BLACK		1
#define V_RED		2
#define V_GREEN		3
#define V_BLUE		4
#define V_CYAN		5
#define V_YELLOW	6
#define V_MAGENTA	7
#define V_LWHITE	8
#define V_LBLACK	9
#define V_LRED		10
#define V_LGREEN	11
#define V_LBLUE		12
#define V_LCYAN		13
#define V_LYELLOW	14
#define V_LMAGENTA	15

#define LT_SOLID		1	/* vsl_type() */
#define LT_LONGDASH		2
#define LT_DOTTED		3
#define LT_DASHDOT		4
#define LT_DASHED		5
#define LT_DASHDOTDOT	6
#define LT_USERDEF		7

#define TA_LEFT		0
#define TA_CENTER	1
#define TA_RIGHT	2
#define TA_BASELINE	0
#define TA_HALF		1
#define TA_ASCENT	2
#define TA_BOTTOM	3
#define TA_DESCENT	4
#define TA_TOP		5

#endif

/* VDI-Calls */
extern int v_opnvwk_screen(void);
extern void vs_gclip(int vhandle, GRECT *rect);
extern void vs_clipoff(int vhandle);
extern void v_rccopy(int vhandle,int x1,int y1,int w,int h,int x2,int y2,int mode);
extern void v_box(short vhandle,short x0,short y0,short x1,short y1);
extern void v_line(short vhandle,short x0,short y0,short x1,short y1);
extern void vst_align(short vhandle,short hor,short vert);

/*************************
 **   special inlines   **
 *************************/

extern long lrotl(long,short);
#ifdef LATTICE
  extern short intel(short);
#else
# define intel(a) intelshort(a)
  extern short intelshort(short);
  extern long intellong(long);
#endif
extern long imul(short,short);
extern short idiv(long,short);
extern short imuldiv(short,short,short);
extern void memcpyw(short*,short*,short);
extern short bset(short,short);
extern long bsetl(long,short);
extern long bset1l(short);
extern short bclr(short,short);
extern long bclrl(long,short);
extern short bchg(short,short);
extern long bchgl(long,short);
extern boolean btst(short,short);
extern boolean btst(short,short);
extern boolean btstl(long,short);
extern short bit(short x);
extern short bitl(long x);

#ifdef LATTICE
#pragma inline d0=lrotl(d0,d1)	{"E3B8";}	/*	rol.l d1,d0		*/
#pragma inline d0=intel(d0)	{"E058";}		/*	ror.w #8,d0	*/
#pragma inline d0=imul(d0,d1)	{"C1C1";}	/*	muls.w d1,d0	*/
#pragma inline d0=idiv(d0,d1)	{"81C1";}	/*	divs.w d1,d0	*/
#pragma inline d0=imuldiv(d0,d1,d2)	{"C1C1";"81C2";}	/*	muls.w d1,d0	divs.w d2,d0	*/
#pragma inline memcpyw(a1,a0,d0)	{"5380";"32D8";"51C8FFFC";}
	/*	subq.l #1,d0	move.w (a0)+,(a1)+	dbra d0,-2	*/
#pragma inline d0=bset(d0,d1)		{"03C0";}	/*	bset d1,d0	*/
#pragma inline d0=bsetl(d0,d1)		{"03C0";}	/*	bset d1,d0	*/
#pragma inline d0=bset1l(d1)		{"7000";"03C0";}	/*	moveq	#0,d0  bset d1,d0	*/
#pragma inline d0=bclr(d0,d1)		{"0380";}	/*	bclr d1,d0	*/
#pragma inline d0=bclrl(d0,d1)		{"0380";}	/*	bclr d1,d0	*/
#pragma inline d0=bchg(d0,d1)		{"0340";}	/*	bchg d1,d0	*/
#pragma inline d0=bchgl(d0,d1)		{"0340";}	/*	bchg d1,d0	*/
#pragma inline d0=btst(d0,d1)	{"0300";"6704";"7001";"6002";"7000";}
#pragma inline d0=btstl(d0,d1)	{"0300";"6704";"7001";"6002";"7000";}
/*	btst d1,d0   beq.s 4   moveq #1,d0   bra.s 2   moveq #0,d0	*/
#endif /* LATTICE */


/*************************************
 **   KeyCode-Konvertierung         **
 *************************************/
/* Statustasten des Systems */
#define KsRSHIFT  0x01
#define KsLSHIFT  0x02
#define KsSH      0x03
#define KsSHIFT   0x03
#define KsCTRL    0x04
#define KsCONTROL 0x04
#define KsALT     0x08
#define KsCAPS    0x10

/* Statustasten nach Konvertierung */
#define KbRSHIFT  0x0100
#define KbLSHIFT  0x0200
#define KbSH      0x0300
#define KbSHIFT   0x0300
#define KbCONTROL 0x0400
#define KbCTRL    0x0400
#define KbALT     0x0800
#define KbCAPS    0x1000
#define KbNUM     0x4000
#define KbSCAN    0x8000

#define KbISO	0x37

/* Scan-Codes fr F-Tasten */
#define KbF1	(KbSCAN|0x3b)
#define KbF2	(KbSCAN|0x3c)
#define KbF3	(KbSCAN|0x3d)
#define KbF4	(KbSCAN|0x3e)
#define KbF5	(KbSCAN|0x3f)
#define KbF6	(KbSCAN|0x40)
#define KbF7	(KbSCAN|0x41)
#define KbF8	(KbSCAN|0x42)
#define KbF9	(KbSCAN|0x43)
#define KbF10	(KbSCAN|0x44)
#define KbF11	(KbSCAN|0x54)
#define KbF12	(KbSCAN|0x55)
#define KbF13	(KbSCAN|0x56)
#define KbF14	(KbSCAN|0x57)
#define KbF15	(KbSCAN|0x58)
#define KbF16	(KbSCAN|0x59)
#define KbF17	(KbSCAN|0x5a)
#define KbF18	(KbSCAN|0x5b)
#define KbF19	(KbSCAN|0x5c)
#define KbF20	(KbSCAN|0x5d)

/* Scan-Codes fr Steuertasten */
#define KbUNDO	(KbSCAN|0x61)
#define KbHELP	(KbSCAN|0x62)
#define KbINSERT	(KbSCAN|0x52)
#define KbDELETE	(KbSCAN|0x53)
#define KbBACKSPACE	0x08
#define KbHOME	(KbSCAN|0x47)
#define KbUP	(KbSCAN|0x48)
#define KbDOWN	(KbSCAN|0x50)
#define KbLEFT	(KbSCAN|0x4b)
#define KbRIGHT	(KbSCAN|0x4d)
#define KbESC	0x1b
#define KbRETURN	0x0d

/* Scancodes fr Alternate und Ziffernblock */
#define KbALT1	(KbSCAN|0x78)
#define KbALT2	(KbSCAN|0x79)
#define KbALT3	(KbSCAN|0x7a)
#define KbALT4	(KbSCAN|0x7b)
#define KbALT5	(KbSCAN|0x7c)
#define KbALT6	(KbSCAN|0x7d)
#define KbALT7	(KbSCAN|0x7e)
#define KbALT8	(KbSCAN|0x7f)
#define KbALT9	(KbSCAN|0x80)
#define KbALT0	(KbSCAN|0x81)

extern unsigned short MapKey(short VDIKey);
/*	76543210 76543210
 *	SxxxACLR CHARCODE	*/

#ifdef __cplusplus
}
#endif

#endif /* _XLB_H */
