/*
 *	eXtended-Library
 *	released for GNU-C
 *	by Volker Hemsen, 03/96
 *	Public Domain / Freeware
 *
 *	Universalroutinen zur Bearbeitung und Verwaltung
 *	von Feldern
 */

#include <macros.h>
#include <stdlib.h>
#include <string.h>
#include <xlb.h>

/* Parameterblock fr Feld anlegen,
   size: Gre eines Elementes, addmem: Anzahl anzuhngender Elemente */
FIELD* FIELD_init(long size,long addmem)
{
	FIELD* f=malloc(sizeof(FIELD));
	if (f) {
		f->address=0;
		f->size=max(size,1);
		f->usedmax=0;
		f->memmax=0;
		f->addmem=max(addmem,4);
		return f;
	}
	return 0;
}

/* Parameterblock fr Feld wieder freigeben */
void FIELD_end(FIELD **f)
{
	if (*f) {
		FIELD_free(*f);
		free(*f);
		*f=0;
	}
}

/* Addresse des Feldelementes pos berechnen */
char* FIELD_address(FIELD *f,long pos)
{
	if (f)
		if ((f->address) && (pos>=0) && (pos<f->usedmax))
			return ((char*)f->address)+f->size*pos;
	return 0;
}

/* Speicher des Feldes freigeben und Variablen zurcksetzen */
void FIELD_free(FIELD *f)
{
	if (f) {
		if (f->address)
			free(f->address);
		f->address=0;
		f->usedmax=0;
		f->memmax=0;
	}
}

/* Speicher fr Feld mit newmax Elementen reservieren
   return: 0: Fehler, !0: alles OK */
boolean FIELD_allocate(FIELD *f,long newmax)
{
	if (f) {
		void *a;
		if (!f->address)
			a=calloc(newmax,f->size);
		else {
			a=realloc(f->address,f->size*newmax);
			{
				long l=newmax-f->memmax;
				if (l>0)
					memclr((char*)a+(f->memmax*f->size),l*f->size);
			}
		}
		if (a) {
			f->address=a;
			f->memmax=newmax;
			return TRUE;
		}
	}
	return FALSE;
}

/* Feldelement an Index pos lschen, den Rest nachrcken
   return: 0:Fehler aufgetreten, !0:alles OK */
boolean FIELD_delete(FIELD *f,long pos)
{
	if (f) {
		if ((f->address) && (pos>=0) && (pos<f->usedmax)) {
			long l=f->usedmax-pos;
			if (l>0)
				memcpy(FIELD_address(f,pos),FIELD_address(f,pos+1),l*f->size);
			memclr(FIELD_address(f,f->usedmax),f->size);
			f->usedmax--;
			return TRUE;
		}
	}
	return FALSE;
}

/* Feldelement an Position pos einfgen und mit Inhalt von
   insert fllen, bzw. bei insert=0 lschen
   return: 0:Fehler aufgetreten, !0:alles OK */
boolean FIELD_insert(FIELD *f,long pos,void *insert)
{
	if (f) {
		short ok;
		if ((!f->address) || (pos>=f->memmax))
			ok=FIELD_allocate(f,max(f->memmax+f->addmem,pos+1));
		else
			ok=TRUE;
		if (ok) {
			char *a;
			long l;
			f->usedmax=max(f->usedmax+1,pos+1);
			a=FIELD_address(f,pos);
			l=f->usedmax-pos+1;
			if (l>=1)
				memcpy(a+f->size,a,l*f->size);
			if (!insert)
				memclr(a,f->size);
			else
				memcpy(a,(char*)insert,f->size);
			return TRUE;
		}
	}
	return FALSE;
}

/* Element ans Ende des Feldes hngen
   return: 0:Fehler, !0:alles OK */
boolean FIELD_add(FIELD *f,void *insert)
{
	if (f)
		return FIELD_insert(f,f->usedmax,insert);
	return FALSE;
}

/* Feld gem Rckgabewert von routine() sortieren,
   die Feldposition curs kann zum Verfolgen eines Elementes
   genutzt werden, oder den Wert 0 erhalten!
   return: 0:es wurde nichts verndert, !0:es wurde etwas verndert! */
boolean FIELD_sort(FIELD *f,long *curs,boolean (*routine)(FIELD *f,long i1,long i2))
{
	boolean sortdone=FALSE;
	if (f) {
		boolean found;
		do {
			register long i;
			found=FALSE;
			for(i=0;i<f->usedmax-1;i++) {
				if (routine(f,i,i+1)) {
					memswp(FIELD_address(f,i),FIELD_address(f,i+1),f->size);
					if (curs) {
						if (i==*curs)
							*curs=i+1;
						else if (*curs==i+1)
							*curs=i;
					}
					found=TRUE;
				}
			}
			sortdone|=found;
		} while (found);
	}
	return sortdone;
}

