/*
 *	eXtended-Library
 *	released for GNU-C
 *	by Volker Hemsen, 03/96
 *	Public Domain / Freeware
 *
 *	mathematischer Parser
 */

#include <xlb.h>
#include <math.h>

#define SYMBLEN	20

#ifndef M_PI
#define M_PI PI
#endif
#ifndef M_E
#define M_E exp(1)
#endif

#define M_c0	2.99792458e8
#define M_gn	9.80665
#define M_gamma	6.672e-11
#define M_Rm	8.31441
#define M_Vmn	22.41383
#define M_NA	6.022045e23
#define M_NL	2.686754e25
#define M_k		1.380662e-23
#define M_SBK	5.67032e-8
#define M_EFK	8.85418782e-12
#define M_MFK	1.25663706144E-6
#define M_F		9.648456e4
#define M_e		1.6021892e-19
#define M_h		6.626176e-34
#define M_me	9.109534e-31
#define M_meu	5.4858026e-4
#define M_mp	1.6726485e-27
#define M_mpu	1.00727647
#define M_mn	1.6749543e-27
#define M_mnu	1.008665012
#define M_u		1.6605655e-27

typedef struct constants {
	char	name[5];
	double	val;
} CONSTANT;

CONSTANT constant[]={
	{ "c0",	M_c0 },
	{ "c",	M_c0 },
	{ "eps0",	M_EFK },
	{ "eps",	M_EFK },
	{ "EFK",	M_EFK },
	{ "e0",	M_e },
	{ "e",	M_E	},
	{ "FK",	M_F },
	{ "F",	M_F },
	{ "",	M_PI },
	{ "gn",	M_gn },
	{ "g",	M_gn },
	{ "",	M_gamma },
	{ "hq",	M_h/2/M_PI },
	{ "h",	M_h	},
	{ "k",	M_k },
	{ "0",	M_MFK },
	{ "",	M_MFK },
	{ "me",	M_me },
	{ "meu",	M_meu },
	{ "MFK",	M_MFK },
	{ "mp",	M_mp },
	{ "mpu",	M_mpu },
	{ "mn",	M_mn },
	{ "mnu",	M_mnu },
	{ "NA",	M_NA },
	{ "NL",	M_NL },
	{ "PI",	M_PI },
	{ "Pi",	M_PI },
	{ "pi",	M_PI },
	{ "Rm",	M_Rm },
	{ "",	M_SBK },
	{ "SBK",	M_SBK },
	{ "u",	M_u },
	{ "Vmn",	M_Vmn },
	{ "",	0 }
};


static double parser_const(char *cmd)
{
	CONSTANT *c=constant;
	while (*c->name) {
		if (!strcmp(c->name,cmd)) {
			*cmd=0;
			return c->val;
		}
		c++;
	}
	return 1;
}


static double parser_docmd(char *cmd,double r,double r1)
{
	switch(*cmd) {
	case 0:
		r=r1;
		break;
	case '*':
		r=r*r1;
		break;
	case '/':
		if (r1==0)
			r=HUGE;
		else
			r=r/r1;
		break;
	case '^':
		if (r==0)
			r=0;
		else
			r=pow(r,r1);
		break;
	case '+':
		r=r+r1;
		break;
	case '-':
		r=r-r1;
		break;
	default:
		strupr(cmd);
		if (!strcmp("SQR",cmd))
			r=pow(r1,2);
		if (!strcmp("SQRT",cmd))
			r=pow(fabs(r1),0.5);
		if (!strcmp("ABS",cmd))
			r=fabs(r1);
		if (!strcmp("TRUNC",cmd))
			r=floor(r1);
		if (!strcmp("ROUND",cmd)) {
			double x=floor(r1);
			if (r1-x<0.5)
				r=x;
			else
				r=ceil(r1);	}
		if (!strcmp("REZI",cmd)) {
			if (r1==0)
				r=HUGE;
			else
				r=1/r1;	}
		if (!strcmp("GRAD",cmd))
			r=r1*180/M_PI;
		if (!strcmp("RAD",cmd))
			r=r1*M_PI/180;
		if (!strcmp("SIN",cmd))
			r=sin(r1);
		if (!strcmp("COS",cmd))
			r=cos(r1);
		if (!strcmp("TAN",cmd)) {
			if (r1==0)
				r=0;
			else
				r=tan(r1);	}
		if ((!strcmp("ASIN",cmd)) || (!strcmp("ARCSIN",cmd)))
			r=asin(r1);
		if ((!strcmp("ACOS",cmd)) || (!strcmp("ARCCOS",cmd)))
			r=acos(r1);
		if ((!strcmp("ATAN",cmd)) || (!strcmp("ARCTAN",cmd)))
			r=atan(r1);
		if (!strcmp("LN",cmd))
			r=log(r1);
		if (!strcmp("LG",cmd))
			r=log10(r1);
		if (!strcmp("EXP",cmd))
			r=exp(r1);
		break;
	}
	cmd[0]=0;
	return r;
}

static char *parser_donumber(char *s,char *cmd1,double *r,double r1)
{
	if (*s=='!') {
		register long i,l=(long)floor(fabs(r1));
		r1=1;
		for(i=1;i<=l;r1*=i++);
		s++;
	}
	*r=parser_docmd(cmd1,*r,r1);
	if (!*cmd1)
		strcpy(cmd1,"*");
	return s;
}

static char *parser_do(char *s,double *ret)
{
	double r=0,r1=0;
	char cmd1[SYMBLEN]="",cmd2[SYMBLEN]="";
	short ende=0;
	while ((*s) && (!ende)) {
		switch (*s) {
			case '+':
			case '-':
				cmd1[0]=*s;
				cmd1[1]=0;
			case '(':
				s++;
				s=parser_do(s,&r1);
				if (*cmd2) {
					if (!*cmd1)
						r=parser_docmd(cmd2,r,r1);
					else {
						r1=parser_docmd(cmd2,r1,r1);
						if (*cmd1)
							r=parser_docmd(cmd1,r,r1);
					}
				}
				else
					r=parser_docmd(cmd1,r,r1);
				break;
			case ')':
				s++;
				ende=(!0);
				break;
			case '/':
			case '*':
			case '^':
				cmd1[0]=*s++;
				cmd1[1]=0;
				break;
			case '#':
			case ' ':
			case ':':
			case '=':
				s++;
				break;
			default:
				if (((*s>='0') && (*s<='9')) || ((*s=='-') && strlen(cmd1))) {
					char *n=s;
					if (*s=='-')
						s++;
					while (((*s>='0') && (*s<='9')) || (*s=='.'))
						s++;
					if ((*s=='E') || (*s=='e')) {
						s++;
						if ((*s=='-') || (*s=='+'))
							s++;
						while ((*s>='0') && (*s<='9'))							
							s++;
					}
					{
						char x=*s;
						*s=0;
						sscanf(n,"%le",&r1);
						*s=x;
						s=parser_donumber(s,cmd1,&r,r1);
					}
				} else {
					char *p=cmd2;
					while (((*s>='a')&&(*s<='z')) || ((*s>='A')&&(*s<='Z'))
							|| ((*s>='')&&(*s<='')) || ((*s>='0')&&(*s<='9')))
						*p++=*s++;
					*p=0;
					if (*s!='(') {
						r1=parser_const(cmd2);
						s=parser_donumber(s,cmd1,&r,r1);
					}
				}
				break;
		}
	}
	*ret=r;
	return s;
}

double parser(char *s)
{
	double r;
	parser_do(s,&r);
	return r;
}

/*
int main(int argc,char *argv[],char *envp[])
{
	int i;
	for(i=1;i<argc;i++)
		printf("Ergebnis Nr.%d = %G\n",i,parser(argv[i]));
	return 0;
}
*/
