/*
 * String functions.
 */
#ifndef _STRING_H
#define _STRING_H

#ifndef _COMPILER_H
#include <compiler.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif


#ifndef _SIZE_T
#define _SIZE_T __SIZE_TYPEDEF__
typedef _SIZE_T size_t;
#endif

#ifndef NULL
#define NULL __NULL
#endif

__EXTERN void *memcpy __PROTO((void *dst, const void *src, size_t size));
__EXTERN void *memmove __PROTO((void *dst, const void *src, size_t size));
__EXTERN int memcmp __PROTO((const void *s1, const void *s2, size_t size));
__EXTERN void *memchr __PROTO((const void *s, int ucharwanted, size_t size));
__EXTERN void *memset __PROTO((void *s, int ucharfill, size_t size));

#ifdef __USE_GNU
/* Copy N bytes of SRC to DEST, return pointer to bytes afterthe last
   written byte.  */
__EXTERN void* __mempcpy __PROTO ((void* __dest, const void* src, size_t n));
__EXTERN void* mempcpy __PROTO ((void* __dest, const void* src, size_t n));
#endif

__EXTERN char *strcpy __PROTO((char *dst, const char *src));
__EXTERN char *strncpy __PROTO((char *dst, const char *src, size_t n));
__EXTERN char *strcat __PROTO((char *dst, const char *src));
__EXTERN char *strncat __PROTO((char *dst, const char *src, size_t n));
__EXTERN int strcmp __PROTO((const char *scan1, const char *scan2));
__EXTERN int strncmp __PROTO((const char *scan1, const char *scan2, size_t n));
__EXTERN int strcoll __PROTO((const char *scan1, const char *scan2));
__EXTERN size_t	strxfrm __PROTO((char *to, const char *from, size_t maxsize));
__EXTERN char *strchr __PROTO((const char *s, int charwanted));
__EXTERN size_t strcspn __PROTO((const char *s, const char *reject));
__EXTERN char *strpbrk __PROTO((const char *s, const char *breakat));
__EXTERN char *strrchr __PROTO((const char *s, int charwanted));
__EXTERN size_t strspn __PROTO((const char *s, const char *accept));
__EXTERN char *strstr __PROTO((const char *s, const char *wanted));
__EXTERN char *strtok __PROTO((char *s, const char *delim));
__EXTERN char *strtok_r __PROTO((char* s, const char* delim, char** saveptr));
__EXTERN size_t strlen __PROTO((const char *scan));
__EXTERN char *strerror __PROTO((int errnum));
/* Provided for compatibility with the GNU libc.  Not really needed
   unless you want to write portable code (the normal strerror function
   is also threadsafe).  */
__EXTERN char *strerror_r __PROTO((int errnum, char* buf, size_t buflen));

#if defined(__USE_SVID) || defined(__USE_BSD)
/* 
 * from henry spencers string lib
 *  these dont appear in ansi draft sec 4.11
 */
__EXTERN void *memccpy __PROTO((void *dst, const void *src, int ucharstop, size_t size));

# ifndef __STRICT_ANSI__
__EXTERN char *strlwr __PROTO((char *string));
__EXTERN char *strupr __PROTO((char *string));
/* CAUTION: there are assumptions elsewhere in the code that strrev()
   reverses in-place
 */
__EXTERN char *strrev __PROTO((char *string));
# endif
#endif

#if defined(__USE_SVID) || defined (__USE_BSD)
__EXTERN char *strdup __PROTO((const char *s));
#endif

/* Return a malloc'd copy of at most N bytes of STRING.  The
   resultant string is terminated even if no null terminator
   appears before STRING[N].  */
#if defined __USE_GNU
__EXTERN char* strndup __PROTO ((const char* __string, size_t __n));
#endif

/*
 * V7 and BSD compatibility.
 */
#ifdef __USE_BSD
__EXTERN char *index __PROTO((const char *s, int charwanted));
__EXTERN char *rindex __PROTO((const char *s, int charwanted));
__EXTERN void bcopy __PROTO((const void *src, void *dst, size_t length));
__EXTERN int bcmp __PROTO((const void *src, const void *dst, size_t n));
__EXTERN void bzero __PROTO((void *b, size_t n));
#endif

__EXTERN void _bcopy __PROTO((const void *src, void *dst, unsigned long length));
__EXTERN int  _bcmp __PROTO((const void *s1, const void *s2, unsigned long length));
__EXTERN void _bzero __PROTO((void *dst, unsigned long length));

#ifdef __USE_MISC
/* FIXME: Do these four lads belong to wany standard?  */
__EXTERN int stricmp __PROTO(( const char *, const char * ));
__EXTERN int strnicmp __PROTO(( const char *, const char *, size_t ));
__EXTERN int strcmpi __PROTO(( const char *, const char * ));
__EXTERN int strncmpi __PROTO(( const char *, const char *, size_t ));
#endif

__EXTERN int strcasecmp __PROTO(( const char *, const char * ));

#ifdef __USE_GNU
__EXTERN int strncasecmp __PROTO(( const char *, const char *, size_t ));

/* Copy SRC to DEST, returning the address of the terminating '\0' in DEST.  */
__EXTERN char* __stpcpy __P ((char* __dest, const char* __src));
__EXTERN char* stpcpy __P ((char* __dest, const char* __src));

/* Copy no more than N characters of SRC to DEST, returning the address of 
   the last character written into DEST.  */
__EXTERN char* __stpncpy __P ((char* __dest, const char* __src, size_t __n));
__EXTERN char* stpncpy __P ((char* __dest, const char* __src, size_t __n));
#endif

/* some macro versions of functions. these are faster, but less
   forgiving of NULLs and similar nasties. to use the library functions,
   just #undef the appropriate things.
*/

#ifdef __GNUC_INLINE__
# ifndef __cplusplus

static __inline__
char *
__strcat(char *dst, const char *src)
{
	register char *_dscan;

	for (_dscan = dst; *_dscan; _dscan++) ;
	while ((*_dscan++ = *src++)) ;
	return dst;
}

static __inline__ 
char *
__strcpy(char *dst, const char *src)
{
	register char *_dscan = dst;
	while ((*_dscan++ = *src++)) ;
	return dst;
}

static __inline__
size_t
__strlen(const char *scan)
{
	register const char *_start = scan+1;
	while (*scan++) ;
	return (size_t)((long)scan - (long)_start);
}

#define strcat 	__strcat
#define strcpy 	__strcpy
#define strlen 	__strlen

# endif /* !__cplusplus */
#endif /* __GNUC_INLINE__ */

#if defined(__LATTICE__) && !defined(_NO_INLINE)

int __builtin_strcmp (const char *, const char *);
char * __builtin_strcpy (char *, const char *);
size_t __builtin_strlen (const char *);

#define strcmp __builtin_strcmp
#define strcpy __builtin_strcpy
#define strlen __builtin_strlen

#endif

#ifdef	__USE_MISC
/* Return the file name within directory of FILENAME.  An equal prototype
   is in <libgen.h>, FIXME!  */
__EXTERN char* basename __PROTO ((__const char *__filename));
#endif

#ifdef __cplusplus
}
#endif

#endif /* _STRING_H */
